import { Component, Directive, Input, Output, HostListener, HostBinding, EventEmitter, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
export var SuiCheckbox = (function () {
    function SuiCheckbox() {
        this.classes = true;
        this.checked = false;
        this.checkChange = new EventEmitter(false);
        this.isDisabled = false;
        this.isReadonly = false;
    }
    Object.defineProperty(SuiCheckbox.prototype, "checkedAttribute", {
        get: function () {
            return this.checked ? "" : null;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiCheckbox.prototype, "isDisabledAttribute", {
        get: function () {
            return this.isDisabled ? "disabled" : null;
        },
        enumerable: true,
        configurable: true
    });
    SuiCheckbox.prototype.onClick = function () {
        if (!this.isDisabled && !this.isReadonly) {
            this.toggle();
        }
    };
    SuiCheckbox.prototype.toggle = function () {
        this.checked = !this.checked;
        this.checkChange.emit(this.checked);
    };
    SuiCheckbox.prototype.writeValue = function (value) {
        var _this = this;
        setTimeout(function () {
            _this.checked = value;
        });
    };
    SuiCheckbox.decorators = [
        { type: Component, args: [{
                    selector: 'sui-checkbox',
                    exportAs: 'suiCheckbox',
                    template: "\n<input class=\"hidden\" type=\"checkbox\" [attr.name]=\"name\" [attr.checked]=\"checkedAttribute\" [attr.disabled]=\"isDisabledAttribute\" [(ngModel)]=\"checked\">\n<label>\n    <ng-content></ng-content>\n</label>\n"
                },] },
    ];
    /** @nocollapse */
    SuiCheckbox.ctorParameters = function () { return []; };
    SuiCheckbox.propDecorators = {
        'classes': [{ type: HostBinding, args: ['class.ui',] }, { type: HostBinding, args: ['class.checkbox',] },],
        'name': [{ type: Input },],
        'checked': [{ type: HostBinding, args: ['class.checked',] },],
        'checkChange': [{ type: Output },],
        'isDisabled': [{ type: Input },],
        'isReadonly': [{ type: HostBinding, args: ['class.read-only',] }, { type: Input },],
        'onClick': [{ type: HostListener, args: ['click',] },],
    };
    return SuiCheckbox;
}());
export var CUSTOM_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(function () { return SuiCheckboxValueAccessor; }),
    multi: true
};
export var SuiCheckboxValueAccessor = (function () {
    function SuiCheckboxValueAccessor(host) {
        this.host = host;
        this.onChange = function () { };
        this.onTouched = function () { };
    }
    SuiCheckboxValueAccessor.prototype.writeValue = function (value) {
        this.host.writeValue(!!value);
    };
    SuiCheckboxValueAccessor.prototype.registerOnChange = function (fn) { this.onChange = fn; };
    SuiCheckboxValueAccessor.prototype.registerOnTouched = function (fn) { this.onTouched = fn; };
    SuiCheckboxValueAccessor.decorators = [
        { type: Directive, args: [{
                    selector: 'sui-checkbox',
                    host: { '(checkChange)': 'onChange($event)' },
                    providers: [CUSTOM_VALUE_ACCESSOR]
                },] },
    ];
    /** @nocollapse */
    SuiCheckboxValueAccessor.ctorParameters = function () { return [
        { type: SuiCheckbox, },
    ]; };
    return SuiCheckboxValueAccessor;
}());
export var SUI_CHECKBOX_DIRECTIVES = [SuiCheckbox, SuiCheckboxValueAccessor];
//# sourceMappingURL=checkbox.js.map