import { Directive, Input, HostBinding, Output, ContentChild, HostListener, ContentChildren } from '@angular/core';
import { DropdownService } from './dropdown.service';
import { SuiDropdownMenu } from './dropdown-menu';
import { KeyCode } from '../util/util';
export var SuiDropdown = (function () {
    function SuiDropdown() {
        this.service = new DropdownService();
    }
    Object.defineProperty(SuiDropdown.prototype, "children", {
        get: function () {
            var _this = this;
            // @ContentChildren includes the current element by default.
            return this._children.filter(function (c) { return c !== _this; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiDropdown.prototype, "isOpenChange", {
        get: function () {
            return this.service.isOpenChange;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiDropdown.prototype, "isActive", {
        get: function () {
            // This is to ensure nested dropdowns don't get made bold.
            return this.service.isOpen && !this.service.isNested;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiDropdown.prototype, "isOpen", {
        get: function () {
            return this.service.isOpen;
        },
        set: function (value) {
            // If we are opening the dropdown, we want to always open its parents.
            this.service.setOpenState(value, !!value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiDropdown.prototype, "isDisabled", {
        get: function () {
            return this.service.isDisabled;
        },
        set: function (value) {
            this.service.setDisabledState(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiDropdown.prototype, "tabIndex", {
        get: function () {
            return (this.isDisabled || this.service.isNested) ? null : 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiDropdown.prototype, "autoClose", {
        get: function () {
            return this.service.autoCloseMode;
        },
        set: function (value) {
            this.service.autoCloseMode = value;
        },
        enumerable: true,
        configurable: true
    });
    SuiDropdown.prototype.ngAfterContentInit = function () {
        var _this = this;
        if (!this._menu) {
            throw new Error("You must set [suiDropdownMenu] on the menu element.");
        }
        this._menu.service = this.service;
        this.childrenUpdated();
        this._children.changes
            .subscribe(function () { return _this.childrenUpdated(); });
    };
    SuiDropdown.prototype.childrenUpdated = function () {
        var _this = this;
        // Reregister child dropdowns each time the menu content changes.
        this.children
            .map(function (c) { return c.service; })
            .forEach(function (s) { return _this.service.registerChild(s); });
    };
    SuiDropdown.prototype.onClick = function (e) {
        // Block the click event from being fired on parent dropdowns.
        e.stopPropagation();
        this.service.toggleOpenState();
    };
    SuiDropdown.prototype.onKeypress = function (e) {
        // Block the keyboard event from being fired on parent dropdowns.
        if (e.keyCode == KeyCode.Enter) {
            e.stopPropagation();
            this.service.setOpenState(true);
        }
    };
    SuiDropdown.decorators = [
        { type: Directive, args: [{
                    selector: '[suiDropdown]'
                },] },
    ];
    /** @nocollapse */
    SuiDropdown.ctorParameters = function () { return []; };
    SuiDropdown.propDecorators = {
        '_menu': [{ type: ContentChild, args: [SuiDropdownMenu,] },],
        '_children': [{ type: ContentChildren, args: [SuiDropdown, { descendants: true },] },],
        'isOpenChange': [{ type: Output },],
        'isActive': [{ type: HostBinding, args: ['class.active',] },],
        'isOpen': [{ type: Input },],
        'isDisabled': [{ type: HostBinding, args: ['class.disabled',] }, { type: Input },],
        'tabIndex': [{ type: HostBinding, args: ['attr.tabindex',] },],
        'autoClose': [{ type: Input },],
        'onClick': [{ type: HostListener, args: ["click", ['$event'],] },],
        'onKeypress': [{ type: HostListener, args: ["keypress", ['$event'],] },],
    };
    return SuiDropdown;
}());
//# sourceMappingURL=dropdown.js.map