import { Directive, Input, ElementRef, ComponentFactoryResolver, ViewContainerRef, HostListener } from '@angular/core';
import { SuiPopup } from './popup';
import { PositioningPlacement } from '../util/positioning.service';
// Creates essentially a 'string' enum.
export var PopupTrigger = {
    Hover: "hover",
    Click: "click",
    OutsideClick: "outsideClick",
    Focus: "focus",
    Manual: "manual"
};
export var SuiPopupDirective = (function () {
    function SuiPopupDirective(_element, _viewContainerRef, _componentFactoryResolver) {
        this._element = _element;
        this._viewContainerRef = _viewContainerRef;
        this._componentFactoryResolver = _componentFactoryResolver;
        this._config = {};
        this.popupTrigger = PopupTrigger.Hover;
        this.popupPlacement = PositioningPlacement.TopLeft;
    }
    Object.defineProperty(SuiPopupDirective.prototype, "popupTemplate", {
        set: function (template) {
            this._config.template = template;
            this.copyConfig();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupHeader", {
        set: function (header) {
            this._config.header = header;
            this.copyConfig();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupText", {
        set: function (text) {
            this._config.text = text;
            this.copyConfig();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupInverted", {
        set: function (inverted) {
            if (typeof inverted == "string") {
                inverted = true;
            }
            this._config.inverted = inverted;
            this.copyConfig();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupBasic", {
        set: function (basic) {
            if (typeof basic == "string") {
                basic = true;
            }
            this._config.basic = basic;
            this.copyConfig();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupTransition", {
        set: function (transition) {
            this._config.transition = transition;
            this.copyConfig();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupTransitionDuration", {
        set: function (duration) {
            this._config.transitionDuration = duration;
            this.copyConfig();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "popupPlacement", {
        set: function (placement) {
            if (!placement) {
                return;
            }
            var _a = placement.split(" "), direction = _a[0], alignment = _a[1];
            var chosenPlacement = [direction];
            switch (alignment) {
                case "top":
                case "left":
                    chosenPlacement.push("start");
                    break;
                case "bottom":
                case "right":
                    chosenPlacement.push("end");
                    break;
            }
            this._placement = chosenPlacement.join("-");
            this.copyConfig();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiPopupDirective.prototype, "_popup", {
        get: function () {
            return this._popupComponentRef.instance;
        },
        enumerable: true,
        configurable: true
    });
    SuiPopupDirective.prototype.copyConfig = function () {
        if (this._popupComponentRef) {
            Object.assign(this._popup.config, this._config);
            if (this.hasOwnProperty("_placement")) {
                this._popup.placement = this._placement;
            }
        }
    };
    SuiPopupDirective.prototype.open = function () {
        var _this = this;
        if (!this._popupComponentRef) {
            var factory = this._componentFactoryResolver.resolveComponentFactory(SuiPopup);
            this._popupComponentRef = this._viewContainerRef.createComponent(factory);
            this._popup.onClose.subscribe(function () {
                _this._popupComponentRef.destroy();
                _this._popupComponentRef = null;
            });
            this.copyConfig();
            this._popup.anchor = this._element;
        }
        this._popup.open();
    };
    SuiPopupDirective.prototype.close = function () {
        this._popup.close();
    };
    SuiPopupDirective.prototype.toggle = function () {
        if (!this._popupComponentRef || (this._popupComponentRef && !this._popup.isOpen)) {
            return this.open();
        }
        return this.close();
    };
    SuiPopupDirective.prototype.onMouseEnter = function () {
        if (this.popupTrigger == PopupTrigger.Hover) {
            this.open();
        }
    };
    SuiPopupDirective.prototype.onMouseLeave = function () {
        if (this.popupTrigger == PopupTrigger.Hover) {
            this.close();
        }
    };
    SuiPopupDirective.prototype.onClick = function () {
        if (this.popupTrigger == PopupTrigger.Click || this.popupTrigger == PopupTrigger.OutsideClick) {
            event.stopPropagation();
            this.toggle();
        }
    };
    SuiPopupDirective.prototype.onDocumentClick = function (e) {
        if (this._popupComponentRef && this.popupTrigger == PopupTrigger.OutsideClick) {
            this.close();
        }
    };
    SuiPopupDirective.prototype.onFocus = function () {
        if (this.popupTrigger == PopupTrigger.Focus) {
            this.open();
        }
    };
    SuiPopupDirective.prototype.onFocusOut = function () {
        if (this.popupTrigger == PopupTrigger.Focus) {
            this.close();
        }
    };
    SuiPopupDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[suiPopup]',
                    exportAs: 'suiPopup'
                },] },
    ];
    /** @nocollapse */
    SuiPopupDirective.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: ViewContainerRef, },
        { type: ComponentFactoryResolver, },
    ]; };
    SuiPopupDirective.propDecorators = {
        'popupTemplate': [{ type: Input },],
        'popupHeader': [{ type: Input },],
        'popupText': [{ type: Input },],
        'popupInverted': [{ type: Input },],
        'popupBasic': [{ type: Input },],
        'popupTransition': [{ type: Input },],
        'popupTransitionDuration': [{ type: Input },],
        'popupPlacement': [{ type: Input },],
        'popupTrigger': [{ type: Input },],
        'onMouseEnter': [{ type: HostListener, args: ["mouseenter",] },],
        'onMouseLeave': [{ type: HostListener, args: ["mouseleave",] },],
        'onClick': [{ type: HostListener, args: ["click",] },],
        'onDocumentClick': [{ type: HostListener, args: ["document:click", ["$event"],] },],
        'onFocus': [{ type: HostListener, args: ["focus",] },],
        'onFocusOut': [{ type: HostListener, args: ["focusout",] },],
    };
    return SuiPopupDirective;
}());
//# sourceMappingURL=popup.directive.js.map