import { Component, Directive, Input, Output, EventEmitter, HostBinding, HostListener, forwardRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
export var SuiRating = (function () {
    function SuiRating() {
        this.ratingClasses = true;
        this._value = 0;
        this._max = 5;
        this.valueChange = new EventEmitter(false);
        this._hoveredIndex = -1;
        this.isReadonly = false;
    }
    Object.defineProperty(SuiRating.prototype, "max", {
        set: function (value) {
            this._max = parseInt(value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiRating.prototype, "icons", {
        get: function () {
            return Array(this._max);
        },
        enumerable: true,
        configurable: true
    });
    SuiRating.prototype.mouseover = function (i) {
        this._hoveredIndex = i;
    };
    SuiRating.prototype.mouseout = function () {
        this._hoveredIndex = -1;
    };
    SuiRating.prototype.click = function (i) {
        if (!this.isReadonly) {
            this._value = i + 1;
            this.valueChange.emit(this._value);
        }
    };
    SuiRating.prototype.writeValue = function (value) {
        this._value = value;
    };
    SuiRating.decorators = [
        { type: Component, args: [{
                    selector: 'sui-rating',
                    exportAs: 'suiRating',
                    template: "\n<i class=\"icon\"\n   *ngFor=\"let icon of icons; let i = index\"\n   (mouseover)=\"mouseover(i)\"\n   (click)=\"click(i)\"\n   [class.selected]=\"_hoveredIndex >= i && !isReadonly\"\n   [class.active]=\"_value > i\">\n</i>\n",
                    styles: ["\n:host.read-only .icon {\ncursor: auto\n}\n"]
                },] },
    ];
    /** @nocollapse */
    SuiRating.ctorParameters = function () { return []; };
    SuiRating.propDecorators = {
        'ratingClasses': [{ type: HostBinding, args: ['class.ui',] }, { type: HostBinding, args: ['class.rating',] },],
        'max': [{ type: Input },],
        'valueChange': [{ type: Output },],
        'mouseout': [{ type: HostListener, args: ['mouseout',] },],
        'isReadonly': [{ type: HostBinding, args: ['class.read-only',] }, { type: Input },],
    };
    return SuiRating;
}());
export var CUSTOM_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(function () { return SuiRatingValueAccessor; }),
    multi: true
};
export var SuiRatingValueAccessor = (function () {
    function SuiRatingValueAccessor(host) {
        this.host = host;
        this.onChange = function () { };
        this.onTouched = function () { };
    }
    SuiRatingValueAccessor.prototype.writeValue = function (value) {
        this.host.writeValue(value);
    };
    SuiRatingValueAccessor.prototype.registerOnChange = function (fn) { this.onChange = fn; };
    SuiRatingValueAccessor.prototype.registerOnTouched = function (fn) { this.onTouched = fn; };
    SuiRatingValueAccessor.decorators = [
        { type: Directive, args: [{
                    selector: 'sui-rating',
                    host: { '(valueChange)': 'onChange($event)' },
                    providers: [CUSTOM_VALUE_ACCESSOR]
                },] },
    ];
    /** @nocollapse */
    SuiRatingValueAccessor.ctorParameters = function () { return [
        { type: SuiRating, },
    ]; };
    return SuiRatingValueAccessor;
}());
export var SUI_RATING_DIRECTIVES = [SuiRating, SuiRatingValueAccessor];
//# sourceMappingURL=rating.js.map