var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
import { Component, ViewContainerRef, ViewChild, Output, EventEmitter, ElementRef, Renderer, forwardRef, Directive } from '@angular/core';
import { SuiSelectBase } from './select-base';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
export var SuiSelect = (function (_super) {
    __extends(SuiSelect, _super);
    function SuiSelect(element, renderer) {
        _super.call(this, element, renderer);
        this.placeholder = "Select one";
        this.selectedOptionChange = new EventEmitter();
    }
    Object.defineProperty(SuiSelect.prototype, "ngModelChange", {
        get: function () {
            // For simplicity we can mirror these two emitters as they do the same thing.
            return this.selectedOptionChange;
        },
        enumerable: true,
        configurable: true
    });
    SuiSelect.prototype.optionsUpdateHook = function () {
        var _this = this;
        if (this._writtenOption && this.options.length > 0) {
            // If there was an value written by ngModel before the options had been loaded, this runs to fix it.
            this.selectedOption = this.options.find(function (o) { return _this._writtenOption == _this.valueGetter(o); });
            if (this.selectedOption) {
                this._writtenOption = null;
                this.drawSelectedOption();
            }
        }
    };
    SuiSelect.prototype.queryUpdateHook = function () {
        // When the query is updated, we just abandon the current selection.
        this.selectedOption = null;
    };
    SuiSelect.prototype.selectOption = function (option) {
        // Choose and emit the selected option.
        this.selectedOption = option;
        this.selectedOptionChange.emit(this.valueGetter(option));
        this.dropdownService.setOpenState(false);
        // The search delay is set to the transition duration to ensure results aren't rendered as the select closes as that causes a sudden flash.
        this.searchService.searchDelay = this._menu.menuTransitionDuration;
        this.searchService.updateQueryDelayed("");
        this.drawSelectedOption();
        // Adds the active property to the items.
        this.onAvailableOptionsRendered();
        // Automatically refocus the search input for better keyboard accessibility.
        this.focusInput();
    };
    SuiSelect.prototype.writeValue = function (value) {
        var _this = this;
        if (value != null) {
            if (this.options.length > 0) {
                // If the options have already been loaded, we can immediately match the ngModel value to an option.
                this.selectedOption = this.options.find(function (o) { return value == _this.valueGetter(o); });
            }
            if (!this.selectedOption) {
                // Otherwise, cache the written value for when options are set.
                this._writtenOption = value;
            }
        }
        this.drawSelectedOption();
    };
    SuiSelect.prototype.initialiseRenderedOption = function (option) {
        _super.prototype.initialiseRenderedOption.call(this, option);
        // Boldens the item so it appears selected in the dropdown.
        option.isActive = option.value == this.selectedOption;
    };
    SuiSelect.prototype.drawSelectedOption = function () {
        if (this.selectedOption && this.optionTemplate) {
            this.drawTemplate(this._optionTemplateSibling, this.selectedOption);
        }
    };
    SuiSelect.decorators = [
        { type: Component, args: [{
                    selector: 'sui-select',
                    template: "\n<i class=\"dropdown icon\"></i>\n<!-- Query input -->\n<input [hidden]=\"!isSearchable\" class=\"search\" type=\"text\" autocomplete=\"off\" [(ngModel)]=\"query\" #queryInput>\n<!-- Placeholder text -->\n<div *ngIf=\"!selectedOption\" class=\"default text\" [class.filtered]=\"!!query\">{{ placeholder }}</div>\n<!-- Selected item -->\n<div class=\"text\" [class.filtered]=\"!!query || !selectedOption\">\n    <span #optionTemplateSibling></span>\n    <span *ngIf=\"!optionTemplate\">{{ labelGetter(selectedOption) }}</span>\n</div>\n<!-- Select dropdown menu -->\n<div class=\"menu\" suiDropdownMenu [menuAutoSelectFirst]=\"isSearchable\">\n    <ng-content></ng-content>\n    <div *ngIf=\"isSearchable && availableOptions.length == 0\" class=\"message\">{{ noResultsMessage }}</div>\n</div>\n"
                },] },
    ];
    /** @nocollapse */
    SuiSelect.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: Renderer, },
    ]; };
    SuiSelect.propDecorators = {
        '_optionTemplateSibling': [{ type: ViewChild, args: ['optionTemplateSibling', { read: ViewContainerRef },] },],
        'selectedOptionChange': [{ type: Output },],
        'ngModelChange': [{ type: Output },],
    };
    return SuiSelect;
}(SuiSelectBase));
// Value accessor for the select.
export var SELECT_VALUE_ACCESSOR = {
    provide: NG_VALUE_ACCESSOR,
    useExisting: forwardRef(function () { return SuiSelectValueAccessor; }),
    multi: true
};
// Value accessor directive for the select to support ngModel.
export var SuiSelectValueAccessor = (function () {
    function SuiSelectValueAccessor(host) {
        this.host = host;
        this.onChange = function () { };
        this.onTouched = function () { };
    }
    SuiSelectValueAccessor.prototype.writeValue = function (value) {
        this.host.writeValue(value);
    };
    SuiSelectValueAccessor.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    SuiSelectValueAccessor.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    SuiSelectValueAccessor.decorators = [
        { type: Directive, args: [{
                    selector: 'sui-select',
                    host: {
                        '(selectedOptionChange)': 'onChange($event)'
                    },
                    providers: [SELECT_VALUE_ACCESSOR]
                },] },
    ];
    /** @nocollapse */
    SuiSelectValueAccessor.ctorParameters = function () { return [
        { type: SuiSelect, },
    ]; };
    return SuiSelectValueAccessor;
}());
//# sourceMappingURL=select.js.map