import { Renderer, ElementRef, Directive, Input, HostBinding } from '@angular/core';
// Possible directions for a transition.
export var TransitionDirection;
(function (TransitionDirection) {
    TransitionDirection[TransitionDirection["In"] = 0] = "In";
    TransitionDirection[TransitionDirection["Out"] = 1] = "Out";
    TransitionDirection[TransitionDirection["Either"] = 2] = "Either";
    TransitionDirection[TransitionDirection["Static"] = 3] = "Static";
})(TransitionDirection || (TransitionDirection = {}));
export var Transition = (function () {
    function Transition(name, duration, direction, onComplete) {
        if (duration === void 0) { duration = 250; }
        if (direction === void 0) { direction = TransitionDirection.Either; }
        if (onComplete === void 0) { onComplete = function () { }; }
        this.type = name;
        if (duration < 1) {
            // We set a minimum duration of 1ms, to give the appearance of an immediate transition whilst allowing positioning calculations to happen without a visible flicker.
            duration = 1;
        }
        this.duration = duration;
        this.direction = direction;
        this.classes = this.type.split(" ");
        this.onComplete = onComplete;
    }
    Object.defineProperty(Transition.prototype, "directionClass", {
        // Converts TransitionDirection to class name.
        get: function () {
            switch (this.direction) {
                case TransitionDirection.In: return "in";
                case TransitionDirection.Out: return "out";
            }
        },
        enumerable: true,
        configurable: true
    });
    return Transition;
}());
export var SuiTransition = (function () {
    function SuiTransition(_renderer, _element) {
        this._renderer = _renderer;
        this._element = _element;
        this.transitionClass = true;
    }
    Object.defineProperty(SuiTransition.prototype, "suiTransition", {
        set: function (tC) {
            // Set the transition controller (e.g. '<div [suiTransition]="transitionController"></div>').
            this.setTransitionController(tC);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiTransition.prototype, "isVisible", {
        get: function () {
            if (this._controller) {
                return this._controller.isVisible;
            }
            return false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(SuiTransition.prototype, "isHidden", {
        get: function () {
            if (this._controller) {
                return this._controller.isHidden;
            }
            return false;
        },
        enumerable: true,
        configurable: true
    });
    // Initialises the controller with the injected renderer and elementRef.
    SuiTransition.prototype.setTransitionController = function (transitionController) {
        this._controller = transitionController;
        this._controller.registerRenderer(this._renderer);
        this._controller.registerElement(this._element.nativeElement);
    };
    SuiTransition.decorators = [
        { type: Directive, args: [{
                    selector: '[suiTransition]',
                    exportAs: 'transition'
                },] },
    ];
    /** @nocollapse */
    SuiTransition.ctorParameters = function () { return [
        { type: Renderer, },
        { type: ElementRef, },
    ]; };
    SuiTransition.propDecorators = {
        'suiTransition': [{ type: Input },],
        'transitionClass': [{ type: HostBinding, args: ['class.transition',] },],
        'isVisible': [{ type: HostBinding, args: ['class.visible',] },],
        'isHidden': [{ type: HostBinding, args: ['class.hidden',] },],
    };
    return SuiTransition;
}());
//# sourceMappingURL=transition.js.map